# -*- coding: utf-8 -*-
"""
Created on Mon Apr 15 17:00:32 2024

@author: Wahb
"""
#%%
import numpy as np
import pandas as pd 
from sklearn.impute import SimpleImputer, KNNImputer
from sklearn.compose import ColumnTransformer
from sklearn.preprocessing import StandardScaler, MinMaxScaler, RobustScaler, OneHotEncoder, LabelEncoder, OrdinalEncoder, Binarizer


# In[1] - Importation de jeu de données
## Chargement des données depuis le fichier Excel
donnees = pd.read_excel(r"C:/Users/2016-1951/Desktop/formation data/IGT.xlsx")              ### à modifier ###

## Affichage des premières lignes pour vérification
print("Aperçu des données avant prétraitement :")
print(donnees.head())

# In[2] - Identification des variables quantitatives & qualitatives
print("Aperçu des types des variables :")
print(donnees.dtypes)

## Identification des noms des variables quantitatives 
numerical = list(donnees.select_dtypes(exclude = "object").columns)

## Identification des noms des variables qualitatives
categorical = list(donnees.select_dtypes(include = "object").columns)

# In[3] - Traitement des valeurs manquantes
## Définition de la statégie d'imputation
imputer = ColumnTransformer([
    ('categorical_transformer', SimpleImputer(strategy='most_frequent'), categorical),      ### à modifier ###
    ('numerical_transformer', SimpleImputer(strategy='mean'), numerical)])                  ### à modifier ###

donnees_imputees = imputer.fit_transform(donnees)

## Facultatif : convertir en DataFrame
donnees_imputees =  pd.DataFrame(data = donnees_imputees, columns = categorical + numerical)

# In[4] -  Encodage
## Codage de la variable cible
etiquettes = ["Bonne", "Mauvaise"] 
encoder = LabelEncoder()
y_enconde = encoder.fit_transform(donnees_imputees["Situation"])                            ### à modifier ###
donnees_imputees["Situation"] =  y_enconde

# In[5] -  Transformation
## Binarisation de la variable 'Total' avec un seuil = 10000

transformeur = Binarizer(threshold = 10000)                                                 ### à modifier ###
parametre_binaire = transformeur.fit_transform(donnees_imputees["Total"].to_frame())        ### à modifier ###

# In[6] - Mise à l'échelle des variables quantitatives 
## Normalisation z-score
donnees_qualitatives = donnees_imputees[categorical]
donnees_quantitatives = donnees_imputees[numerical].astype(float) 

## Définition de la normalisation à utiliser
scaler = StandardScaler()                                                                   ### à modifier ###           

## Données normalisées                                                              
donnees_quantitatives_normalisees = scaler.fit_transform(donnees_quantitatives)   

## Facultatif : convertir en DataFrame
donnees_quantitatives_normalisees =  pd.DataFrame(data = donnees_quantitatives_normalisees, columns = numerical)

# In[7] - Sélection des caractéristiques
## Matrice de corrélation
dimension_donnees = donnees_quantitatives.shape[1]
matrice_correlation = np.zeros((dimension_donnees, dimension_donnees))
for i in range(dimension_donnees):
    for j in range(dimension_donnees):
        matrice_correlation[i,j] = np.corrcoef(donnees_quantitatives.iloc[:,i], donnees_quantitatives.iloc[:,j])[0,1]  
        
## Facultatif : convertir la matrice de corrélation en DataFrame 
matrice_correlation =  pd.DataFrame(data = matrice_correlation, columns = numerical, index = numerical)   

# In[8] - Exportation de données sous Excel
donnes_a_exporter = pd.DataFrame(data = donnees_quantitatives_normalisees)                   ### à modifier ###     
donnes_a_exporter.to_excel("IGT preprocessed.xlsx", index = False)                          ### à modifier ###